import math
pi = math.pi
from scipy.optimize import fsolve
from sympy import *

def Bshift(startingPolygon, mids, length, delta, h, bShift):
    """
    Calculates the B-shift for a given shape, h and edge length
    https://archive.bridgesmathart.org/2018/bridges2018-515.pdf
    """

    n = len(startingPolygon.vertices)
    t = pi*(n - 2)/n
    l = startingPolygon.sides[0].length
    shape_h = math.tan(t/2)/2*(length*delta*(1 - h))
    scale = h

    alpha = t/2
    redPolygon = startingPolygon.scale(scale, scale, pt = startingPolygon.centroid)

    midPt1 = mids[0]
    midPt2 = mids[-1]

    #[AB,BC,thetaA,thetaB]
    def func(x):

        """
        Function for the system defined in the paper.  
        Stores only the length of AB from the solution      
        """

        return [x[0] + x[1] - AC_flat,
         x[2] + x[3] + theta_C - pi,
         AC_folded*math.sin(theta_C) - x[0]*math.sin(x[3]),
         x[1]*math.sin(theta_C) - x[0]*math.sin(x[2])]

    #alpha > 45
    if alpha - pi/4 > .2:
 
        A = Point(((midPt1[0] + midPt2[0])/2,(midPt1[1] + midPt2[1])/2))
        C = startingPolygon.vertices[0]
        AC_flat = float(A.distance(C))
        AC_parr = float(A.distance(redPolygon.vertices[0]))
        hc = shape_h/math.tan(alpha)
        AC_perp = shape_h - hc
        AC_folded = float(math.sqrt(AC_perp**2 + AC_parr**2))
        gamma = float(math.atan(AC_parr/AC_perp))
        theta_C = pi - gamma - alpha

        root = fsolve(func, [.5,.5,0,0])
        bShift[n] = root[0]
       
    #alpha = 45
    elif abs(alpha - pi/4) < .1:

        A = Point(((midPt1[0] + midPt2[0])/2,(midPt1[1] + midPt2[1])/2))
        C = startingPolygon.vertices[0]
        AC_flat = float(A.distance(C))
        AC_parr = float(A.distance(redPolygon.vertices[0]))
        hc = shape_h
        AC_folded = AC_parr
        theta_C = pi/2 - alpha

        root = fsolve(func, [.5,.5,0,0])
        bShift[n] = root[0]

    #Alpha < 45
    else:

        A = Point(((midPt1[0] + midPt2[0])/2,(midPt1[1] + midPt2[1])/2))
        C = startingPolygon.vertices[0]
        AC_flat = float(A.distance(C))
        AC_parr = float(A.distance(redPolygon.vertices[0]))
        hc = float(shape_h/math.tan(alpha))
        AC_perp = hc - shape_h
        AC_folded = float(math.sqrt(AC_perp**2 + AC_parr**2))
        gamma = float(math.atan(AC_perp/AC_parr))
        theta_C = pi/2 - gamma - alpha

        root = fsolve(func, [.5,.5,0,0])
        bShift[n] = root[0]  