import math
pi = math.pi
from dxfwrite import DXFEngine as dxf
from sympy import Polygon
import time
from sympy import *
from bshift import Bshift

def generate(drawing, combined, cycles, connections, length, h, delta, totalTime, print_tiling, print_creases):
    #Use this for crease patterns
    border_color, tiling_color, start_color, reduced_color, midpoint_color, leg_color, corner_leg_color, sides_color, connect_color, corner_connect_color = [20,7,1,1,5,1,5,5,1,5]

    def mag(p):
        """
        Finds the magnitude of points p1 and p2
        """

        return math.sqrt(p[0]**2 + p[1]**2)

    def findRotation(check, checkCent):

        """
        Finds the rotation of the polygon
        relative to the first polygon of that
        type found
        """
        pt = (check[0] - checkCent[0], check[1] - checkCent[1])
        if pt[0] == 0:

            if pt[1] > 0:

                return pi/2

            else:

                return 3*pi/2

        elif pt[0] > 0:

            return math.atan(pt[1]/pt[0]) + 2*pi

        else:

            return math.atan(pt[1]/pt[0]) + 3*pi

    def addPolygon(poly,c):
        
        """
        Draws the polygon poly on the dxf file in the
        color c
        """

        previous = poly[0]

        for i in range(1,len(poly)):

            current = poly[i]
            drawing.add(dxf.line((previous[0],previous[1]),(current[0], current[1]), color = c))
            combined.add(dxf.line((previous[0],previous[1]),(current[0], current[1]), color = c))
            previous = current

        drawing.add(dxf.line((poly[0][0],poly[0][1]),(poly[-1][0], poly[-1][1]), color = c))
        combined.add(dxf.line((poly[0][0],poly[0][1]),(poly[-1][0], poly[-1][1]), color = c))



    print("Adding face creases")
    start = time.process_time()

    #Finds all cycles that share an edge
    #and all cycles that share a vertex
    edgeAdjacentCycles = {}
    vertexAdjacentCycles = {}
    for i in range(len(cycles)):

        c1 = tuple(cycles[i])
        edgeAdjacentCycles[c1] = []
        vertexAdjacentCycles[c1] = []

        for j in cycles[i:]:

            c2 = tuple(j)
            if len(set(c1).intersection(set(c2))) == 2:

                edgeAdjacentCycles[c1].append(c2)
                vertexAdjacentCycles[c1].append(c2)
        
    perpsInFace = {}
    vertsInFace = {}
    shapeCreases = {}
    bShift = {}


    #Enumerates through every face to add crease patterns
    for face in cycles:

        sumx, sumy = 0,0
        for pt in face:

            sumx += pt[0] 
            sumy += pt[1]

        cent = (sumx/len(face), sumy/len(face))

        angle = round(findRotation(face[0], cent) % (2*pi/len(face)),1)
        
        if len(face) != 3 and angle == 1:

            angle = 0

        elif len(face) == 3 and angle == 2.1:

            angle = 0

        n = len(face)
        t = pi*(n - 2)/n
        scale = delta

        #If the face is the first polygon of this type and angle
        #creates a new crease pattern
        if (len(face),angle) not in shapeCreases:
            
            #Adds the crease pattern to be used for identical faces
            shapeCreases[len(face),angle] = [[(face[0][0] - cent[0], face[0][1] - cent[1]),(face[1][0] - cent[0], face[1][1] - cent[1])]]    

            #Created a Polygon object for every
            #cycle found that was not sifted out
            outerPolygon = Polygon(*face)
            outerVerts = outerPolygon.vertices

            #Creates the starting polygon using
            #the outer polygon and delta
            startVerts = []
            for pt in outerPolygon.vertices:

                startVerts.append(((scale * (pt[0] - cent[0])) + cent[0], (scale * (pt[1] - cent[1])) + cent[1]))

            startingPolygon = Polygon(*startVerts)
            startingVerts = startingPolygon.vertices
        
            #Saves every vertice in startingVerts to
            #be used for the corner connection creases
            for v in startingVerts:

                if tuple(face) not in vertsInFace.keys():

                    vertsInFace[tuple(face)] = [v]

                else:

                    vertsInFace[tuple(face)].append(v)
    
            #Draws the starting polygon
            addPolygon(startingPolygon.vertices,start_color)

            #Adds the crease pattern to be used for identical faces
            adjustedStartVerts = [(v[0] - cent[0], v[1] - cent[1]) for v in startingPolygon.vertices]
            shapeCreases[(len(face),angle)].append((adjustedStartVerts,start_color))

            #Creates the reduced polygon using 
            #the starting polygon and h
            scale = h
            redVerts = []
            for pt in startingPolygon.vertices:
            
                redVerts.append(((scale * (pt[0] - cent[0])) + cent[0], (scale * (pt[1] - cent[1])) + cent[1]))
            
            reducedPolygon = Polygon(*redVerts)
            reducedVerts = reducedPolygon.vertices
            
            #Finds the midpoint polygon using the
            #midpoints of every edge in the 
            #reduced polygon
            midVerts = []
            previous = reducedVerts[0]

            for i in range(1, len(reducedVerts)):

                current = reducedVerts[i]
                midVerts.append(((current[0] + previous[0])/2,(current[1] + previous[1])/2))
                previous = current

            midVerts.append(((reducedVerts[0][0] + reducedVerts[-1][0])/2, (reducedVerts[0][1] + reducedVerts[-1][1])/2))
            addPolygon(midVerts, midpoint_color)

            #Adds the crease pattern to be used for identical faces
            adjustedMidVerts = [(v[0] - cent[0], v[1] - cent[1]) for v in midVerts]
            shapeCreases[(len(face),angle)].append((adjustedMidVerts,midpoint_color))

            midPoly = Polygon(*midVerts)

            startingLines = startingPolygon.sides
            reducedLines = reducedPolygon.sides

            #Adds the leg and side creases
            for i in range(len(reducedLines)):
            
                inner1 = reducedLines[i]
                inner2 = reducedLines[i - 1]
                outer1 = startingLines[i]
                outer2 = startingLines[i - 1]
                mid1 = midVerts[i-1]
                mid2 = midVerts[i]
                mid3 = ((mid1[0] + mid2[0])/2,(mid1[1] + mid2[1])/2)
                point1 = inner1.p1
                point2 = inner2.p2

                inter1 = inner1.perpendicular_line(point1).intersection(outer1)
                inter2 = inner2.perpendicular_line(point2).intersection(outer2)
    
                #Saves points to be used for connection creases
                if tuple(face) not in perpsInFace.keys():

                    perpsInFace[tuple(face)] = [(inter1[0][0], inter1[0][1])]

                else:

                    perpsInFace[tuple(face)].append((inter1[0][0], inter1[0][1]))

                if tuple(face) not in perpsInFace.keys():

                    perpsInFace[tuple(face)] = [(inter2[0][0], inter2[0][1])]

                else:

                    perpsInFace[tuple(face)].append((inter2[0][0], inter2[0][1]))


                #If there is no B-shift value for a shape
                #one is generated and stored in bShift
                if n not in bShift:

                    Bshift(startingPolygon, midVerts, length, delta, h, bShift)

                midLine = Line((midVerts[i][0],midVerts[i][1]),(midVerts[i-1][0], midVerts[i-1][1]))
                
                dx = midVerts[i][0] - midVerts[i-1][0]
                dy = midVerts[i][1] - midVerts[i-1][1]
                num = -dx
                den = dy
                magnitude = math.sqrt(num**2 + den**2)
            
                #Finds the point B using the point A and
                #the given length of AB
                B = (mid3[0] + den*bShift[n]/magnitude, mid3[1] + num*bShift[n]/magnitude)

                if mag((B[0] - startingPolygon.vertices[i][0],B[1] - startingPolygon.vertices[i][1])) >= mag((mid3[0] - startingPolygon.vertices[i][0],mid3[1] - startingPolygon.vertices[i][1])):
            
                    B = (mid3[0] - den*bShift[n]/magnitude, mid3[1] - num*bShift[n]/magnitude)

                #dx = mid3[0] - startingPolygon.vertices[i][0]
                #dy = mid3[1] - startingPolygon.vertices[i][1]

                #magnitude = sqrt(dx**2 + dy**2)

                #B = (startingPolygon.vertices[i][0] + bShift[n]*dx/magnitude, startingPolygon.vertices[i][1] + bShift[n]*dy/magnitude)
            
                #if mag((B[0] - startingPolygon.vertices[i][0],B[1] - startingPolygon.vertices[i][1])) >= mag((mid3[0] - startingPolygon.vertices[i][0],mid3[1] - startingPolygon.vertices[i][1])) and len(cycle) == 3:
            
                #   B = (startingPolygon.vertices[i][0] - bShift[n]*dx/magnitude, startingPolygon.vertices[i][1] - bShift[n]*dy/magnitude)
                
                #Draws the reduced polygon
                drawing.add(dxf.line((mid1[0], mid1[1]),(B[0], B[1]), color = reduced_color))
                drawing.add(dxf.line((mid2[0], mid2[1]),(B[0], B[1]), color = reduced_color))
                combined.add(dxf.line((mid1[0], mid1[1]),(B[0], B[1]), color = reduced_color))
                combined.add(dxf.line((mid2[0], mid2[1]),(B[0], B[1]), color = reduced_color))

                #Adds the crease pattern to be used for identical faces
                shapeCreases[(len(face),angle)].append(([mid1[0] - cent[0], mid1[1] - cent[1]], [mid2[0] - cent[0], mid2[1] - cent[1]], [B[0] - cent[0], B[1] - cent[1]], reduced_color))
                
                #Draws the non-diagonal legs
                drawing.add(dxf.line((inter1[0][0],inter1[0][1]),(B[0], B[1]), color = leg_color))
                drawing.add(dxf.line((inter2[0][0],inter2[0][1]),(B[0], B[1]), color = leg_color))
                combined.add(dxf.line((inter1[0][0],inter1[0][1]),(B[0], B[1]), color = leg_color))
                combined.add(dxf.line((inter2[0][0],inter2[0][1]),(B[0], B[1]), color = leg_color))

                shapeCreases[(len(face),angle)].append(([inter1[0][0] - cent[0], inter1[0][1] - cent[1]], [inter2[0][0] - cent[0], inter2[0][1] - cent[1]], leg_color))

                #Draws the diagonal leg
                drawing.add(dxf.line((startingVerts[i][0], startingVerts[i][1]),(B[0], B[1]), color = corner_leg_color))
                combined.add(dxf.line((startingVerts[i][0], startingVerts[i][1]),(B[0], B[1]), color = corner_leg_color))

                shapeCreases[(len(face),angle)].append(([startingVerts[i][0] - cent[0], startingVerts[i][1] - cent[1]], corner_leg_color))

                #Draws the sides
                drawing.add(dxf.line((midVerts[i][0], midVerts[i][1]),(inter1[0][0], inter1[0][1]), color = sides_color))
                drawing.add(dxf.line((midVerts[i-1][0], midVerts[i-1][1]),(inter2[0][0], inter2[0][1]), color = sides_color))
                combined.add(dxf.line((midVerts[i][0], midVerts[i][1]),(inter1[0][0], inter1[0][1]), color = sides_color))
                combined.add(dxf.line((midVerts[i-1][0], midVerts[i-1][1]),(inter2[0][0], inter2[0][1]), color = sides_color))
            
                shapeCreases[(len(face),angle)].append(([midVerts[i][0] - cent[0], midVerts[i][1] - cent[1]], [midVerts[i-1][0] - cent[0], midVerts[i-1][1] - cent[1]], [inter1[0][0] - cent[0], inter1[0][1] - cent[1]], [inter2[0][0] - cent[0],inter2[0][1] - cent[1]], sides_color))
        
        #If a polygon of this type and rotation
        #has already been generated then the
        #crease pattern is pasted instead of
        #generating it again
        else:

            adjustedStart = [(v[0] + cent[0],v[1] + cent[1]) for v in shapeCreases[(len(face),angle)][1][0]]
            addPolygon(adjustedStart,shapeCreases[(len(face),angle)][1][1])

            #Saves every vertice in startingVerts to
            #be used for the corner connection creases
            for v in adjustedStart:

                if tuple(face) not in vertsInFace.keys():

                    vertsInFace[tuple(face)] = [v]

                else:

                    vertsInFace[tuple(face)].append(v)
        
            adjustedMid = [(v[0] + cent[0],v[1] + cent[1]) for v in shapeCreases[(len(face),angle)][2][0]]
            addPolygon(adjustedMid,shapeCreases[(len(face),angle)][2][1])

            for i in range(len(face)):

                mid1, mid2, B, reduced_color = shapeCreases[(len(face),angle)][3 + 4*i][0], shapeCreases[(len(face),angle)][3 + 4*i][1], shapeCreases[(len(face),angle)][3 + 4*i][2], shapeCreases[(len(face),angle)][3][3]
                
                drawing.add(dxf.line((mid1[0] + cent[0], mid1[1] + cent[1]),(B[0] + cent[0], B[1] + cent[1]), color = reduced_color))
                drawing.add(dxf.line((mid2[0] + cent[0], mid2[1] + cent[1]),(B[0] + cent[0], B[1] + cent[1]), color = reduced_color))
                combined.add(dxf.line((mid1[0] + cent[0], mid1[1] + cent[1]),(B[0] + cent[0], B[1] + cent[1]), color = reduced_color))
                combined.add(dxf.line((mid2[0] + cent[0], mid2[1] + cent[1]),(B[0] + cent[0], B[1] + cent[1]), color = reduced_color))
        
                inter1, inter2, leg_color = shapeCreases[(len(face),angle)][4 + 4*i][0], shapeCreases[(len(face),angle)][4 + 4*i][1], shapeCreases[(len(face),angle)][4 + 4*i][2]
                drawing.add(dxf.line((inter1[0] + cent[0],inter1[1] + cent[1]),(B[0] + cent[0], B[1] + cent[1]), color = leg_color))
                drawing.add(dxf.line((inter2[0] + cent[0],inter2[1] + cent[1]),(B[0] + cent[0], B[1] + cent[1]), color = leg_color))
                combined.add(dxf.line((inter1[0] + cent[0],inter1[1] + cent[1]),(B[0] + cent[0], B[1] + cent[1]), color = leg_color))
                combined.add(dxf.line((inter2[0] + cent[0],inter2[1] + cent[1]),(B[0] + cent[0], B[1] + cent[1]), color = leg_color))
            
                if tuple(face) not in perpsInFace.keys():

                    perpsInFace[tuple(face)] = [(inter1[0] + cent[0], inter1[1] + cent[1])]

                else:

                    perpsInFace[tuple(face)].append((inter1[0] + cent[0], inter1[1] + cent[1]))

                if tuple(face) not in perpsInFace.keys():

                    perpsInFace[tuple(face)] = [(inter2[0] + cent[0], inter2[1] + cent[1])]

                else:

                    perpsInFace[tuple(face)].append((inter2[0] + cent[0], inter2[1] + cent[1]))

                startingVerts, corner_leg_color = shapeCreases[(len(face),angle)][5 + 4*i][0], shapeCreases[(len(face),angle)][5 + 4*i][1]
                drawing.add(dxf.line((startingVerts[0] + cent[0], startingVerts[1] + cent[1]),(B[0] + cent[0], B[1] + cent[1]), color = corner_leg_color))
                combined.add(dxf.line((startingVerts[0] + cent[0], startingVerts[1] + cent[1]),(B[0] + cent[0], B[1] + cent[1]), color = corner_leg_color))
        
                midVerts1, midVerts2, inter1, inter2, sides_color = shapeCreases[(len(face),angle)][6 + 4*i][0], shapeCreases[(len(face),angle)][6 + 4*i][1], shapeCreases[(len(face),angle)][6 + 4*i][2], shapeCreases[(len(face),angle)][6 + 4*i][3], shapeCreases[(len(face),angle)][6 + 4*i][4]
                drawing.add(dxf.line((midVerts1[0] + cent[0], midVerts1[1] + cent[1]),(inter1[0] + cent[0], inter1[1] + cent[1]), color = sides_color))
                drawing.add(dxf.line((midVerts2[0] + cent[0], midVerts2[1] + cent[1]),(inter2[0] + cent[0], inter2[1] + cent[1]), color = sides_color))
                combined.add(dxf.line((midVerts1[0] + cent[0], midVerts1[1] + cent[1]),(inter1[0] + cent[0], inter1[1] + cent[1]), color = sides_color))
                combined.add(dxf.line((midVerts2[0] + cent[0], midVerts2[1] + cent[1]),(inter2[0] + cent[0], inter2[1] + cent[1]), color = sides_color))
            
    print(time.process_time() - start)
    print("Adding connection creases")
    start = time.process_time()
    deltaDict = {}

    #Finds every possible length of a "valley"
    #between two shapes, using all pairs of
    #shapes in connections
    scale = delta
    for connection in connections:

        t1 = pi*(connection[0] - 2)/connection[0]
        d1 = float((length - scale*length)*math.tan(t1/2)/2)
        t2 = pi*(connection[1] - 2)/connection[1]
        d2 = float((length - scale*length)*math.tan(t2/2)/2)
        if (connection[0], connection[1]) not in deltaDict.keys():

            deltaDict[(connection[0], connection[1])] = d1 + d2

        if (connection[1], connection[0]) not in deltaDict.keys():

            deltaDict[(connection[1], connection[0])] = d1 + d2



    #Adds all connection creases by checked all cycles that share an edge
    for key in edgeAdjacentCycles.keys():

        for checkCycle in edgeAdjacentCycles[key]:

            for first in perpsInFace[key]:

                for second in perpsInFace[checkCycle]:
        
                    #If the distance between the perpendicular points is
                    #the correct distance for the two shapes that should
                    #be connected then the points are connected
                    distance = math.sqrt((first[0] - second[0])**2 + (first[1] - second[1])**2)
                        
                    if (len(key), len(checkCycle)) in deltaDict.keys():

                        delta = deltaDict[(len(key), len(checkCycle))]

                    elif (len(checkCycle), len(key)) in deltaDict.keys():

                        delta = deltaDict[(len(checkCycle), len(key))]

                    else:

                        continue

                    correct_distance = abs(distance - delta) <= .001

                    if correct_distance: 

                        drawing.add(dxf.line((first[0], first[1]),(second[0], second[1]), color = connect_color))       
                        combined.add(dxf.line((first[0], first[1]),(second[0], second[1]), color = connect_color))                         
            
                        break



    print(time.process_time() - start)
    print("Adding corner connection creases")
    start = time.process_time()

    #Adds all corner connection creases that share a vertex
    for key in vertexAdjacentCycles.keys():

        for checkCycle in vertexAdjacentCycles[key]:

            for first in vertsInFace[key]:

                for second in vertsInFace[checkCycle]:

                    #If the distance between the corner connected points 
                    #is the correct distance for the two shapes that 
                    #should be connected then the points are connected
                    distance = math.sqrt((first[0] - second[0])**2 + (first[1] - second[1])**2)

                    if (len(key), len(checkCycle)) in deltaDict.keys():

                        delta = deltaDict[(len(key), len(checkCycle))]

                    elif (len(checkCycle), len(key)) in deltaDict.keys():

                        delta = deltaDict[(len(checkCycle), len(key))]

                    else:

                        continue

                    correct_distance = abs(distance - delta) <= .001

                    if correct_distance: 

                        drawing.add(dxf.line((first[0], first[1]),(second[0], second[1]), color = corner_connect_color))    
                        combined.add(dxf.line((first[0], first[1]),(second[0], second[1]), color = corner_connect_color))                            
                    
                        break



    print(time.process_time() - start)
    print("Total time")
    print(time.process_time() - totalTime)

    drawing.save()
    if print_tiling and print_creases:
        combined.save()