import math
pi = math.pi
from dxfwrite import DXFEngine as dxf
import time
from sympy import *

def generate(allAngleLists, pairings, drawing, combined, length, t_first, t_last, y_bottom, y_top, file_name, print_tiling, print_creases):
    #Use this for crease patterns
    border_color, tiling_color, start_color, reduced_color, midpoint_color, leg_color, corner_leg_color, sides_color, connect_color, corner_connect_color = [20,7,1,1,5,1,5,5,1,5]

    #(x,y) --> [number of edges, edge #, incoming angle, previous crossing type, current crossing type]
    crossings = {}

    #Run the initial crossing separately because it does not have a previous crossing
    firstCross = [True]

    graph = {}
    def draw_cross(x,y):

        """
        Draws a crossing with n edges around the point (x,y)
        """

        #Only runs for the first crossing
        if firstCross[0]:

            #Always assumes the first crossing is the first crossing type in angle list
            angleList = allAngleLists[0]

            #Number of edges depends on the number of edges in that specific crossing
            n = len(angleList)

            #Loops through every edge in the crossing type
            for edge in range(n):

                #Finds the new x,y values using the angle for that specific edge
                new_x = round(x + length*math.cos(angleList[edge]),3)
                new_y = round(y + length*math.sin(angleList[edge]),3)

                #Checks if the new point is within the boundaries and not full already
                if draw_line_possible(new_x,new_y):

                    #Adds the new line to the drawing
                    if print_tiling:
                        drawing.add(dxf.line((x,y),(new_x, new_y), color = tiling_color))
                        combined.add(dxf.line((x,y),(new_x, new_y), color = tiling_color))

                    #Creates the digraph to enumerate faces
                    if (x,y) not in graph:

                        graph[(x,y)] = [(new_x,new_y)]

                    else:

                        graph[(x,y)].append((new_x,new_y))

                    if (new_x,new_y) not in graph:

                        graph[(new_x,new_y)] = [(x,y)]

                    else:

                        graph[(new_x,new_y)].append((x,y))

                    #Updates the crossings dictionary for the new point
                    increment_crossings(new_x,new_y,edge,(angleList[edge])%(2*pi),0,pairings[(0,edge)][0])

            #Once the loop exits, the crossing at (x,y) is guaranteed to be full
            crossings[(x,y)][0] = n

            #Updates firstCross so draw_cross loops normally
            firstCross[0] = False

        #Loop only runs this, after the first crossing is drawn
        else:

            incomingEdge = crossings[(x,y)][1]
            incomingAngle = crossings[(x,y)][2]
            previousCrossing = crossings[(x,y)][3]
            newCrossingType = crossings[(x,y)][4]

            #
            #Finds the crossing and edge to be connected
            #using the incoming edge and the previous crossing
            pairedEdge = pairings[(previousCrossing, incomingEdge)][1]
            angleList = allAngleLists[newCrossingType]

            #Number of crossings depends on the number edges in that specific crossing
            n = len(angleList)

            #Finds the angle that would form a straight line with the incoming angle
            #then finds the difference between that angle and the angle of the
            #paired edge to find the angle the new crossing needs to be rotated by
            delta = (pi + incomingAngle) - angleList[pairedEdge]

            #Loops through every edge in the crossing type
            for edge in range(n):

                #Finds the new x,y values using the angle for that specific edge
                new_angle = delta + angleList[edge]
                new_x = round(x + length*math.cos(new_angle),3)
                new_y = round(y + length*math.sin(new_angle),3)

                #Checks if the new point is within the boundaries and not full already
                if draw_line_possible(new_x,new_y):

                    #Adds the new line to the drawing
                    if print_tiling:
                        drawing.add(dxf.line((x,y),(new_x, new_y), color = tiling_color))
                        combined.add(dxf.line((x,y),(new_x, new_y), color = tiling_color))

                    #Creates the digraph to enumerate faces
                    if (x,y) not in graph:

                        graph[(x,y)] = [(new_x,new_y)]
                        
                    else:

                        graph[(x,y)].append((new_x,new_y))

                    if (new_x,new_y) not in graph:

                        graph[(new_x,new_y)] = [(x,y)]

                    else:

                        graph[(new_x,new_y)].append((x,y))

                    #Updates the crossings dictionary for the new point
                    increment_crossings(new_x, new_y, edge, (new_angle) % (2*pi), newCrossingType, pairings[(newCrossingType,edge)][0])

            #Once the loop exits, the crossing at (x,y) is guaranteed to be full
            crossings[(x,y)][0] = n

    def increment_crossings(x,y,incomingEdge,incomingAngle,crossingType,previousCross):

        """
        Increments existing elements of crossings by one.
        New points are added to the dictionary with
        the number of edges as 1
        """

        if (x,y) in crossings:

            crossings[(x,y)][0] = crossings[(x,y)][0] + 1

        else:

            crossings[(x,y)] = [1,incomingEdge,incomingAngle,crossingType,previousCross]

    def draw_line_possible(x,y):

        """
        Checks if a point (x,y) is within the drawn boundaries and is not already a full crossing
        """

        #n depends on the crossing type so n is set to the number
        #of edges in that specific crossing
        if (x,y) in crossings:

            currentCrossing = crossings[(x,y)][4]
            n = len(allAngleLists[currentCrossing])

        #Checks if (x,y) is in crossings first, to avoid a key error when checking
        #if it is full yet
        notFull = not (x,y) in crossings or crossings[(x,y)][0] < n
        return within_boundary(x,y) and notFull

    def within_boundary(x,y):
        
        """
        Checks if a point (x,y) is within the drawn boundaries
        """

        rightBound = x <= t_last
        leftBound = x >= t_first
        upBound = y <= y_top
        belowBound = y >= y_bottom
        return rightBound and leftBound and upBound and belowBound

    def notComplete():

        """
        Checks if there are any points in crossings that can still have edges added
        """

        for x,y in crossings.keys():

            current_crossing = crossings[(x,y)][4]
            number_of_edges = crossings[(x,y)][0]
            n = len(allAngleLists[current_crossing])

            #If there is a incomplete point, immediately return
            #to avoid checking unnecesarily
            if number_of_edges < n:
                return True

        return False

    #Adds the initial crossing to the crossings dictionary, with
    #previous crossing set to -1 because it does not have a
    #previous crossing
    crossings[(round(t_first + length,3), round(y_bottom + length,3))] = [1,0,0,-1,0]

    #Draws the initial cross
    draw_cross(round(t_first + length,3), round(y_bottom + length,3))

    #Loops until there are no crossings that can be added
    print("Generating tiling")
    start = time.process_time()
    while notComplete():

        #Dictionaries cannot be modified during iteration
        #so makes a list of the keys to iterate through instead
        keys = list(crossings.keys())

        #Loops through every (x,y) currently in crossings
        for k in keys:

            current_crossing = crossings[(k[0],k[1])][4]
            current_number_of_edges = crossings[(k[0],k[1])][0]
            n = len(allAngleLists[current_crossing])

            #Only draws a cross for non-crossingsd vertices
            if  crossings[(k[0],k[1])][0] < n:    
                
                draw_cross(k[0],k[1])

    #Saves the tiling dxf file
    if print_tiling:

        drawing.save()

    print(time.process_time() - start)

    #Uncomment to save a .txt file of every crossing
    #output_file = open(file_name.replace('dxf','') + 'txt',"wt")
    #for key in crossings.keys():
    #    output_file.write("{} {} ".format(key[0],key[1]))
    #output_file.close()

    if not print_creases:

        exit()
    return graph, crossings
