import math
pi = math.pi
from dxfwrite import DXFEngine as dxf
import PySimpleGUI as sg
import time
import tilings
import generate_tiling
import enumerate_faces
import generate_creases

"""
Run on command line as

'python kuniform.py'

Recommended:
Edge Length = 1
Horizonatal Size * Vertical Size <= 10,000
Delta Proportion = .8
h proportion = .6

"""

#Creates input screen
sg.theme('DarkAmber')

layout = [  
            [sg.Text('Number of vertices')],
            [sg.Combo([
                '1 Uniform ({})'.format(len(tilings.getDict('1 Uniform'))),
                '2 Uniform ({})'.format(len(tilings.getDict('2 Uniform'))),
                '3 Uniform ({})'.format(len(tilings.getDict('3 Uniform'))),
                '4 Uniform ({})'.format(len(tilings.getDict('4 Uniform'))), 
                '5 Uniform ({})'.format(len(tilings.getDict('5 Uniform'))),
                '6 Uniform ({})'.format(len(tilings.getDict('6 Uniform'))), 
                '7 Uniform ({})'.format(len(tilings.getDict('7 Uniform'))), 
                '8 Uniform ({})'.format(len(tilings.getDict('8 Uniform'))), 
                '9 Uniform ({})'.format(len(tilings.getDict('9 Uniform'))),
                '10 Uniform ({})'.format(len(tilings.getDict('10 Uniform'))),
                ], size=(43, 1))],
            [sg.Button('Ok'), sg.Button('Cancel')]
        ]

window = sg.Window('K-Uniform Tilings and Creasing', layout)
while True:
    event, values = window.read()
    if not '' in values.values():
        numberOfVertices = values[0].split()[0] + ' ' + values[0].split()[1]
        break      
    if event == sg.WIN_CLOSED or event == 'Cancel': # if user closes window or clicks cancel
        print("Closing program")
        exit()

window.close()

tilingDict = tilings.getDict(numberOfVertices)
print_tiling = False
print_creases = False
layout = [  
            [sg.Text('{} tilings'.format(numberOfVertices))],
            [sg.Combo([*list(tilingDict.keys())], size=(43, 1))],
            [sg.Text('File name (Omit file extension)')], [sg.InputText()],
            [sg.Text('Edge length (Integer)')], [sg.InputText()],
            [sg.Text('Horizontal size (Integer)')], [sg.InputText()],
            [sg.Text('Vertical size (Integer)')], [sg.InputText()],
            [sg.Text('Delta proportion (Float)')], [sg.InputText()],
            [sg.Text('h proportion (Float)')], [sg.InputText()],
            [sg.CB('Print tiling?')],[sg.CB('Print crease pattern?')],
            [sg.Button('Ok'), sg.Button('Cancel')] ]

window = sg.Window('K-Uniform Tilings and Creasing', layout)
while True:
    event, values = window.read()
    #print(values)
    if not '' in values.values():

        tiling = tilingDict[values[0]]
        pairings = tiling[0]
        allAngleLists = tiling[1]

        file_name = values[1]

        length = int(values[2])

        t_first = 0
        t_last = int(values[3])

        y_bottom = 0
        y_top = int(values[4])

        delta = float(values[5])

        h = float(values[6])

        print_tiling = values[7]

        print_creases = values[8]

        break

    if event == sg.WIN_CLOSED or event == 'Cancel':
        print("Closing program")
        exit()
    
window.close()



totalTime = time.process_time()

#Finds every shape included in the tiling
shapes_included = []
for angles in allAngleLists:

    previous = angles[0]
    sum = 0

    for i in range(1, len(angles)):

        current = angles[i]
        angle = current - previous
        sum = sum + angle
        shape = round(2*pi/(pi - angle),0)

        if shape not in shapes_included:

            shapes_included.append(int(shape))

        previous = current

    angle = 2*pi - sum
    shape = round(2*pi/(pi - angle),0)

    if shape not in shapes_included:

        shapes_included.append(int(shape))

shapes_included.sort()

#Finds which shapes share an edge in the tiling
connections = []
for angleList in allAngleLists:
    aFirst = angleList[0]
    aSecond = angleList[1]
    for aThird in angleList[2:]:

        shapeFirst = abs(aFirst - aSecond)
        shapeSecond = abs(aSecond - aThird)

        shapeFirst = int(round(2*(shapeFirst)/(pi - shapeFirst) + 2,0))
        shapeSecond = int(round(2*(shapeSecond)/(pi - shapeSecond) + 2,0))
      
        if (shapeFirst, shapeSecond) not in connections and (shapeSecond, shapeFirst) not in connections:
            connections.append((shapeFirst, shapeSecond))
        
        aFirst = aSecond
        aSecond = aThird

    aFirst = angleList[-2]
    aSecond = angleList[-1]
    aThird = angleList[0]

    shapeFirst = abs(aFirst - aSecond)
    shapeSecond = abs(aSecond - aThird)

    shapeFirst = int(round(abs(2*(shapeFirst)/(pi - shapeFirst) + 2),0))
    shapeSecond = int(round(abs(2*(shapeSecond)/(pi - shapeSecond) + 2),0))
    
    if (shapeFirst, shapeSecond) not in connections and (shapeSecond, shapeFirst) not in connections:
            connections.append((shapeFirst, shapeSecond))

    aFirst = angleList[-1]
    aSecond = angleList[0]
    aThird = angleList[1]

    shapeFirst = abs(aFirst - aSecond)
    shapeSecond = abs(aSecond - aThird)

    shapeFirst = int(round(abs(2*(shapeFirst)/(pi - shapeFirst) + 2),0))
    shapeSecond = int(round(abs(2*(shapeSecond)/(pi - shapeSecond) + 2),0))
    
    if (shapeFirst, shapeSecond) not in connections and (shapeSecond, shapeFirst) not in connections:
            connections.append((shapeFirst, shapeSecond))

#Use this for crease patterns
border_color, tiling_color, start_color, reduced_color, midpoint_color, leg_color, corner_leg_color, sides_color, connect_color, corner_connect_color = [20,7,1,1,5,1,5,5,1,5]

#border_color, tiling_color, start_color, reduced_color, midpoint_color, leg_color, corner_leg_color, sides_color, connect_color, corner_connect_color = [20,7,7,7,7,7,7,7,7,7]
#border_color, tiling_color, start_color, reduced_color, midpoint_color, leg_color, sides_color, connect_color = [20,random.randint(1,255),random.randint(1,255),random.randint(1,255),random.randint(1,255),random.randint(1,255),random.randint(1,255),random.randint(1,255)]

print("Shapes used: ",shapes_included)

#Prints the delta and h values for each shape given the scale
#factors for h and delta
for shape in shapes_included:
    t = pi*(shape-2)/shape
    shape_delta = round(math.tan(t/2)/2*(length*(1 - delta)),3)
    shape_h = round(math.tan(t/2)/2*(length*delta*(1 - h)),3)
    print("For n = {}, delta = {}, h = {}".format(shape,shape_delta,shape_h))

#Creates dxf file for the tiling and adds a box around the border
drawing = dxf.drawing('{}_tiling.dxf'.format(file_name))

drawing.add(dxf.line( (t_first,y_bottom), (t_first,y_top), color=border_color))
drawing.add(dxf.line( (t_first,y_top), (t_last,y_top), color=border_color))
drawing.add(dxf.line( (t_last,y_bottom), (t_last,y_top), color=border_color))
drawing.add(dxf.line( (t_first,y_bottom), (t_last,y_bottom), color=border_color))

#Creates dxf file for the tiling and crease pattern and adds a box around the border
combined = dxf.drawing('{}_combined.dxf'.format(file_name))

combined.add(dxf.line( (t_first,y_bottom), (t_first,y_top), color=border_color))
combined.add(dxf.line( (t_first,y_top), (t_last,y_top), color=border_color))
combined.add(dxf.line( (t_last,y_bottom), (t_last,y_top), color=border_color))
combined.add(dxf.line( (t_first,y_bottom), (t_last,y_bottom), color=border_color))


graph, crossings = generate_tiling.generate(allAngleLists, pairings, drawing, combined, length, t_first, t_last, y_bottom, y_top, file_name, print_tiling, print_creases)

cycles = enumerate_faces.enumerate(graph, crossings, shapes_included)

#Creates dxf file for the crease pattern and adds a box around the border
drawing = dxf.drawing('{}_creased.dxf'.format(file_name))

drawing.add(dxf.line( (t_first,y_bottom), (t_first,y_top), color=border_color))
drawing.add(dxf.line( (t_first,y_top), (t_last,y_top), color=border_color))
drawing.add(dxf.line( (t_last,y_bottom), (t_last,y_top), color=border_color))
drawing.add(dxf.line( (t_first,y_bottom), (t_last,y_bottom), color=border_color))

generate_creases.generate(drawing, combined, cycles, connections, length, h, delta, totalTime, print_tiling, print_creases)